/**
 * \file: main.cpp
 *
 * Főprogram az óceánmodell teszteléséhez.
 *
 */

#include <iostream>
#include "ocean.h"
#include "halak.h"
using namespace std;

/// Összegző sablon.
/// Iterátor alapján bejárja a tárolót és
/// megszámolja, hogy hány olyan van, ami megfelel a
/// predikátumnak
template<class Iter, class Pred>
int szamol(Iter elso, Iter utso, Pred pred) {
    int db = 0;
    while (elso != utso)
        if (pred(*elso++)) db++;
    return db;
}

/// Halneveket összehasonlító fv.objektum
struct HalnevCmp {
    char refnev;            // referencia név

    /// Konstruktor letárolja a referencia karaktert
    /// @param nev - referencianév
    HalnevCmp(char nev) :refnev(nev) {}

    /// Referencia összehasonlítása
    /// @param o - Obj objektum
    /// @return - true, ha az objektum neve megegyezik a referencianévvel
    bool operator()(const Obj* o) const {
        return o->getnev() == refnev;
    }
};

int main() {
    Ocean atlanti;

    atlanti.replObj(Koord(0,0), new Hal);	// első hal
    atlanti.replObj(Koord(5,10), new Hal);	// újabb
    atlanti.replObj(Koord(MaxN-1, MaxM-1), new Hal);
    atlanti.replObj(Koord(2,5), new Capa);	// cápa
    char cont = 'N';
    for (int i = 0; i < 52; i++) {
       atlanti.rajzol(cout);

	   // A szamol template argumentumai levezethetők igy nem kell kírni
       std::cout << "(Hal:" << szamol(atlanti.begin(), atlanti.end(), HalnevCmp('H'));

       // de ki is írhatnánk...
       std::cout << ", Capa:" << szamol<Ocean::Iterator, HalnevCmp>(atlanti.begin(), atlanti.end(),
                                                                   HalnevCmp('C')) << ")" << endl;
       if (cont != 'I') {
         std::cin >> noskipws >> cont;
       }
       atlanti.lep();
    }
}

