#ifndef OBJ_H
#define OBJ_H

/**
 * \file: obj.h
 *
 * Obj Alapobjektum.
 *
 */
#include "memtrace.h"
#include "koord.h"
#include "ocean.h"

class Ocean;		/// Elődeklaráció. A körkörös hivatkozás miatt kell

/// összehasonlító függvény típusa
typedef bool cmpf_t(int, int);

/// alapértelmezés szerint kisebbet (gyengébbet) keres
inline bool kisebb(int a, int b) { return a < b; }

/// Õs objektum.
class Obj {
protected:
    char nev;       /// Objektum neve
    int meret;      /// nagyobb megeszi a kisebbet
    int iter;       /// Iteráció számlálója
    bool ittaveg;   /// megszűnést jelző flag
public:
	/// Konstruktor:
	/// @param nev - név 1 char
	/// @param meret - hal mérete (ereje)
    Obj(char nev, int meret = 0)
        :nev(nev), meret(meret), iter(0), ittaveg(false) {}

    /// Név lekérdezése
    /// @return - nev
    char getnev() const { return nev; }

    /// Méret lekérdezése
    /// @return - meret
    char getmeret() const { return meret; }

    /// ittaveg flag - ha az objektumot meg kell semmisíteni
    /// @return - ittaveg
    bool is_vege() const { return ittaveg; }

    /// Kisebb/nagyobb hal vagy víz keresése. Először halat keres.
    /// @param pos - kezdő pozíció
    /// @param oc - óceán
    /// @param cmp - predikátum a méret összehasonlításához
    /// @return - megtalált pozíció - ha nincs, érvénytelen
    Koord keres(const Koord& pos, Ocean& oc, cmpf_t cmp = kisebb) const;

    /// Viselkedést megvalósító függvény
    /// Nem absztrakt, hogy ne kelljen megvalósítani mindenütt
	/// @param pos - pozíció
    /// @param oc - óceán
	/// @param it - iterációs lépésszám
	virtual void lep(const Koord& pos, Ocean& oc, int it) {}

	/// Destruktor:
	/// Virtuális, hogy a leszármazottaknak is meghívódjon
	virtual ~Obj() {}
};

#endif

