/**
 * \file: ocean.cpp
 *
 * Ocean objektum megvalstsa.
 *
 */

#include <stdexcept>
#include "ocean.h"
#include "halak.h"
using namespace std;

/// Minden cellt vzzel "tlt fel"
Ocean::Ocean() :iter(0) {
    for (int i = 0; i < MaxN; i++)
        for (int j = 0; j < MaxM; j++)
            cellak[i][j] = new Viz;
}

/// Megszntet minden trolt objektumot
Ocean::~Ocean() {
    for (int i = 0; i < MaxN; i++)
        for (int j = 0; j < MaxM; j++)
            delete cellak[i][j];
}

/// Cellban trolt pointer lekrdezse.
/// @param pos - cellapozci
/// @return objektum pointere
Obj* Ocean::getObj(Koord pos) const {
    return cellak[pos.i][pos.j];
}

/// Cellban trolt pointer bersa.
/// @param pos - cellapozci
/// @param o - j objektum pointere
void Ocean::setObj(Koord pos, Obj* o) {
    cellak[pos.i][pos.j] = o;
}

/// Cellban trolt pointer kicsrlse.
/// A rgi objektum felszabadul
/// @param pos - cellapozci
/// @param o - j objektum pointere
void Ocean::replObj(Koord pos, Obj* o) {
    delete cellak[pos.i][pos.j];
    cellak[pos.i][pos.j] = o;
}

/// Cellarcsok "kirajzolsa"
/// @param os - output steram
void Ocean::rajzol(std::ostream& os) const {
    os << iter << "." << std::endl;
    for (int i = 0; i < MaxN; i++) {
        for (int j = 0; j < MaxM; j++)
            os << cellak[i][j]->getnev();
        os << std::endl;
    }
}

/// Egy itercis lps
void Ocean::lep() {
    iter++;
    for (int i = 0; i < MaxN; i++)
        for (int j = 0; j < MaxM; j++) {
            Koord pos(i,j);
            cellak[i][j]->lep(pos, *this, iter);
            if (cellak[i][j]->is_vege())
                replObj(pos, new Viz);
        }
}


/// Ocean Iterator Pre inkremens
Ocean::Iterator& Ocean::Iterator::operator++() {
    if (p == 0 || p == pe)
        throw out_of_range("Iterator++");
    p++;
    return *this;
}

/// Ocean Iterator Post inkremens
Ocean::Iterator Ocean::Iterator::operator++(int) {
    Iterator tmp = *this;
    if (p == 0 || p == pe)
        throw out_of_range("Iterator++");
    p++;
    return tmp;
}

/// Ocean Iterator Csillag
Obj* Ocean::Iterator::operator*() {
    if (p == 0 || p == pe)
        throw out_of_range("Iterator*");
    return *p;
}

/// Ocean Iterator Nyil operator
Obj** Ocean::Iterator::operator->() {
     if (p == 0 || p == pe)
         throw out_of_range("Iterator->");
     return p;
}
