/**
 * \file: ocean.h
 *
 * Ocean objektum deklarációja
 *
 *
 */

#ifndef OCEAN_H
#define OCEAN_H

#include <iostream>
#include "obj.h"

/// Óceán mérete.
const int MaxN = 10;
const int MaxM = 40;

class Obj;		// Elődeklaráció. (A körkörös include miatt kell.)

/// Ocean objektum.
/// Statikus méretű cellarácsot tartalmaz.
/// Minden cella egy objektum mutatóját tárolja.
class Ocean {
    int iter;                   /// Iteráció számlálója
    Obj *cellak[MaxN][MaxM];    /// Cellák tárolója
public:
	class Iterator;             // elődeklaráció

	/// Deafult konstruktor
    Ocean();

    /// Eldönti egy pozícióról, hogy az érvényes óceán pozíció-e
    /// @return - true, ha érvényes
    bool ervenyes(const Koord& k) const {
        return k.i >= 0 && k.i < MaxN && k.j >= 0 && k.j < MaxM;
	}

    /// Cellában tárolt pointer lekérdezése.
    /// @param pos - cellapozíció
    /// @return objektum pointere
    Obj* getObj(Koord pos) const;

    /// Cellában tárolt pointer beírása.
    /// @param pos - cellapozíció
    /// @param o - új objektum pointere
    void setObj(Koord pos, Obj* o);

    /// Cellában tárolt pointer kicserélése.
    /// A régi objektum felszabadul
    /// @param pos - cellapozíció
    /// @param o - új objektum pointere
    void replObj(Koord pos, Obj* o);

    /// Cellarácsok "kirajzolása"
    /// @param os - output steram
    void rajzol(std::ostream& os) const;

    /// Egy iterációs lépés
    void lep();

    /// Kezdőérték
    Iterator begin() { return Iterator(*this); }

    /// Végérték
    Iterator end() { return Iterator(*this, MaxN*MaxM); }

    /// Óceán iterátora
    class Iterator {
        Obj **p;        // aktuális pointer
        Obj **pe;       // végpointer
    public:
        /// Inicializálatlan iterátor
        Iterator() :p(0), pe(0) {}

        /// Konstruktor a begin-hez és az end-hez
        /// @param o - óceán
        /// @param n - akt. index
        Iterator(Ocean& o, int n = 0)
            :p(&o.cellak[0][0]+n), pe(&o.cellak[0][0]+MaxN*MaxM) {}

        /// !=
        bool operator!=(Iterator& it) {
            return p != it.p;
        }

        /// ==
        bool operator==(Iterator& it) {
            return p == it.p;
        }

        /// Pre inkremens
        Iterator& operator++();

        /// Post inkremens
        Iterator operator++(int);

        /// Csillag
        Obj* operator*();

        /// Nyíl operátor
        Obj** operator->();
    };

    ~Ocean();
};
#endif
