/**
 * \file: gen_array_iter.hpp
 *
 * Generikus tömb iterátorral.
 *  Előadáson bemutatott változat kis kiegészítéssel (inic, i++, ->)
 *    Az osztály definícióját és a tagfüggvények deklarációit
 *    nem tesszük külön fájlba, mivel a fordításnál mindkettőnek
 *    jelen kell lennie!
 */
#ifndef GEN_ARRAY_ITER
#define GEN_ARRAY_ITER

#include <stdexcept>

/// Array sablon
template<class T, unsigned int maxsiz = 6>
class Array {
	T t[maxsiz];            // elemek tömbje
public:
    // minden default tagfüggvény jó, nem kell átdefiniálni
    class iterator;	        // elõdeklaráció

    /// Konstruktor T-ket tartalmazó tömbből
    /// @param p  - poninter T elemeket tartalmazó tömb elejére
    /// @param pe - poninter a tömb utolsó eleme után
    Array(T *p, T*pe) {	    // két pointer közötti elemekbõl inicializaló konstruktor
	for (unsigned int i = 0; p != pe && i < maxsiz; i++)
	    t[i] = *p++;
    }

    /// Iterátor létrehozása az elejére
    iterator begin() {      // létrehoz egy iterátort és az elejére állítja
        return iterator(*this);
    }

    /// Iterátor létrehozása a végére
    iterator end() {        // létrehozza és az utolsó elem után állítja
        return iterator(*this, maxsiz);
    }

    /// iterator osztály
    class iterator {
        T *p, *pe;	        // pointer az akt elemre, és az utolsó utánira
    public:
        // Van benne pointer, mégis jó a default másoló konstruktor és az értékadó op. Miért ?

        /// Konstruktorok
        iterator() :p(0), pe(0) {}  // default kontruktor
        iterator(Array& a, int ix = 0)
            :p(a.t+ix), pe(a.t+maxsiz) {} // kontruktor Array-ról

        /// Pre-inkremens
        iterator& operator++() {    // növeli az iterátort (pre)
            if (p != pe) ++p;
            return *this;
        }

        /// Post-inkemens
        iterator operator++(int) {  // növeli az iterátort (post)
                                    // nem referencia értékû (miért ?)
            iterator tmp = *this;
            if (p != pe) p++;
            return tmp;
        }

        /// operator!=
        bool operator!=(const iterator &i) { // összehasonlít
            return(p != i.p);
        }

        /// operator*
        T& operator*() {            // indirekció
            if (p != pe) return *p;
            else throw std::out_of_range("Array: Hibas indirekcio");
        }

        /// operator->
        T* operator->() {           // indirekció
            if (p != pe) return p;
            else throw std::out_of_range("Array: Hibas indirekcio");
        }

    };  // iterator osztály vége
}; // generikus tömb vége

#endif
