/**
 * \file: generikus_lista_iter.cpp
 *
 * Előadáson bemutatott generikus lista iterátorral
 *
 */

#include <stdexcept>

#ifndef GENERIKUS_LISTA_ITER_HPP
#define GENERIKUS_LISTA_ITER_HPP

template<class T>
class Lista  {
   struct ListaElem {	// privát struktúra
     T  adat;		    // adat
     ListaElem *kov;	// pointer a következőre
     ListaElem(ListaElem *p = NULL) :kov(p) {}
     // kihasználjuk, hogy pointert másol a másoló
     ListaElem& operator=(const ListaElem); // véletlenül se használjuk
   };
   ListaElem *elso;	    // pointer az elsőre
   ListaElem *akt;	    // pointer az aktuálisra
   Lista(const Lista&); // nem használható
   Lista& operator=(const Lista&); // nem használható
      bool hasonlit(const T d1, T d2) { return(d1<d2); }
public:
   Lista() { akt = elso = new ListaElem;}	// strázsa létrehozása
   void beszur(const T& dat);      	// elem beszúrása
   class iterator;		// elődeklaráció
   iterator begin() {   // létrehoz egy iterátort és az elejére állítja
     return(iterator(*this));
   }
   iterator end() {     // létrehozza és az utolsó elem után állítja
     return(iterator());
   }
   class iterator {		// belső osztály, a publikus részek kilátszanak
     ListaElem *akt;	// mutató az aktuális elemre
   public:
	 iterator() : akt(NULL) {};   // végére állítja az iterátort
 	 iterator(const Lista& l) : akt(l.elso) { // elejére állítja
	   if (akt->kov == NULL)
         akt = NULL;    // strázsa miatti trükk
     }
	 iterator& operator++() { // növeli az iterátort (pre)
       if (akt != NULL) {
         akt = akt->kov;  	     // következőre
         if (akt->kov == NULL)
           akt = NULL;  // strázsa miatti trükk
       }
       return(*this);
     }
     iterator operator++(int) { // növeli az iterátort (post)
       iterator tmp = *this;    // előző érték
       operator++();	// növel
       return(tmp);	    // előzővel kell visszatérni
     }
     bool operator!=(const iterator &i) const { // összehasonlít
       return(akt != i.akt);
	 }
     T& operator*() {	// indirekció
       if (akt != NULL ) return(akt->adat);
       else throw std::out_of_range("Hibás");
     }
	 T* operator->() {	// indirekció
       if (akt != NULL) return(&akt->adat);
       else throw std::out_of_range("Hibás");
     }
   }; // iterator osztály vége
   ~Lista();
};

template <class T>          // tagfüggvénysablon
Lista<T>::~Lista() {
	ListaElem *p;			// futó pointer
    while ((p = elso) != NULL) {
        elso = p->kov;
        delete p;
    }
}

template <class T>          // tagfüggvénysablon
void Lista<T>::beszur(const T& dat) {
	ListaElem *p;			// futó pointer
    for (p = elso; p->kov != NULL && hasonlit(p->adat, dat); p = p->kov);
	ListaElem *uj = new ListaElem(*p); //régit másol
    p->adat = dat; p->kov = uj; // adat beírása
}

#endif // GENERIKUS_LISTA_ITER_HPP
